@@ -1,5 +1,9 @@
 # -*- coding: utf-8 -*-
-from django.conf.urls.defaults import patterns, url
+try:
+    from django.conf.urls import patterns, url
+except ImportError:
+    # Backward compatibility for Django prior to 1.6
+    from django.conf.urls.defaults import patterns, url
 from django_qbe.exports import formats
 
 urlpatterns = patterns('django_qbe.views',
